{****************************************************************************}
{*                                                                          *}
{*  Dateiname:     DZVINFO.PAS                                              *}
{*                                                                          *}
{*  Beschreibung:  Delphi-Komponente TDZVersionInfo                         *}
{*                                                                          *}
{*                 TDZVersionInfo ist eine Komponente zum Auslesen von      *}
{*                 Versionsinformationen aus Programmen und DLLs.           *}
{*                                                                          *}
{*                 Wird die Eigenschaft 'FileName' leergelassen, liefert    *}
{*                 die Komponente die Versionsinformationen des eigenen     *}
{*                 Programms (Projekt|Optionen|Versionsinfo).               *}
{*                                                                          *}
{*                 Eigenschaften:                                           *}
{*                 InfoAvailable        - Versionsinfo verfgbar?           *}
{*                 FileVersionNumber    - Dateiversion (Record)             *}
{*                 ProductVersionNumber - Produktversion (Record)           *}
{*                 IsDebug              - Debug-Modus?                      *}
{*                 IsPreRelease         - Vorab-Version?                    *}
{*                 IsPrivateBuild       - Privates Build?                   *}
{*                 IsSpecialBuild       - Spezielles Build?                 *}
{*                 LanguageCode         - Sprachen-Code                     *}
{*                 CharsetCode          - Zeichensatz-Code                  *}
{*                 Language             - Sprache                           *}
{*                 Comments             - Kommentare                        *}
{*                 CompanyName          - Firmenname                        *}
{*                 FileDescription      - Dateibeschreibung                 *}
{*                 FileVersion          - Dateiversion (String)             *}
{*                 InternalName         - Interner Name                     *}
{*                 LegalCopyright       - Copyright-Vermerk                 *}
{*                 LegalTrademarks      - Warenzeichen                      *}
{*                 OriginalFilename     - Original-Dateiname                *}
{*                 PrivateBuild         - Beschreibung Privates Build       *}
{*                 ProductName          - Produktname                       *}
{*                 ProductVersion       - Produktversion                    *}
{*                 SpecialBuild         - Beschreibung Spezielles Build     *}
{*                 FileName             - Pfadname der Datei, aus der die   *}
{*                                        Versionsinfo ausgelesen werden    *}
{*                                        soll                              *}
{*                                                                          *}
{*  Autor:         Dietmar Zucker                                           *}
{*                 Frankenstrae 5                                          *}
{*                 D-78048 Villingen-Schwenningen                           *}
{*                                                                          *}
{*                 dietmar.zucker@t-online.de                               *}
{*                 http://home.t-online.de/home/dietmar.zucker              *}
{*                                                                          *}
{*  Copyright:     (c) 2001 Dietmar Zucker                                  *}
{*                                                                          *}
{****************************************************************************}


unit DZVInfo;

interface

uses
  Windows, Classes;

type
  TVersionNumber = record
    V1: word;
    V2: word;
    V3: word;
    V4: word;
  end;

type
  TDZVersionInfo = class(TComponent)
  private
    FFileName: string;
    FPInfoBuffer: PChar;
    FFixedInfo: TVSFixedFileInfo;
    FLanguageCode: word;
    FCharsetCode: word;
    FInfoAvailable: boolean;
    procedure ReadVersionInfo;
    function GetStringFileInfo( Index: integer ): string;
    procedure SetFileName( Value: string );
    function GetFileVersionNumber: TVersionNumber;
    function GetProductVersionNumber: TVersionNumber;
    function GetLanguageCode: word;
    function GetCharsetCode: word;
    function GetLanguage: string;
    function GetIsDebug: boolean;
    function GetIsPreRelease: boolean;
    function GetIsPrivateBuild: boolean;
    function GetIsSpecialBuild: boolean;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    property InfoAvailable: boolean read FInfoAvailable;
    property FileVersionNumber: TVersionNumber read GetFileVersionNumber;
    property ProductVersionNumber: TVersionNumber read GetProductVersionNumber;
    property IsDebug: boolean read GetIsDebug;
    property IsPreRelease: boolean read GetIsPreRelease;
    property IsPrivateBuild: boolean read GetIsPrivateBuild;
    property IsSpecialBuild: boolean read GetIsSpecialBuild;
    property LanguageCode: word read GetLanguageCode;
    property CharsetCode: word read GetCharsetCode;
    property Language: string read GetLanguage;
    property Comments: string index 0 read GetStringFileInfo;
    property CompanyName: string index 1 read GetStringFileInfo;
    property FileDescription: string index 2 read GetStringFileInfo;
    property FileVersion: string index 3 read GetStringFileInfo;
    property InternalName: string index 4 read GetStringFileInfo;
    property LegalCopyright: string index 5 read GetStringFileInfo;
    property LegalTrademarks: string index 6 read GetStringFileInfo;
    property OriginalFileName: string index 7 read GetStringFileInfo;
    property PrivateBuild: string index 8 read GetStringFileInfo;
    property ProductName: string index 9 read GetStringFileInfo;
    property ProductVersion: string index 10 read GetStringFileInfo;
    property SpecialBuild: string index 11 read GetStringFileInfo;
  published
    property FileName: string read FFileName write SetFileName;
  end;

procedure Register;

implementation

uses
  SysUtils, Forms;

type
  PTranslation = ^TTranslation;
  TTranslation = record
    Lang:    word;
    Charset: word;
  end;

const
  SFINames: array [0..11] of string = (
    'Comments', 'CompanyName', 'FileDescription', 'FileVersion',
    'InternalName', 'LegalCopyright', 'LegalTrademarks', 'OriginalFileName',
    'PrivateBuild', 'ProductName', 'ProductVersion', 'SpecialBuild'
  );

{****************************************************************************}
{*                                                                          *}
{*  Name:          Create()                                                 *}
{*                                                                          *}
{*  Beschreibung:  Konstruktor der TDZVersionInfo-Klasse                    *}
{*                                                                          *}
{*  Input:         AOwner - Besitzer der Objekt-Instanz                     *}
{*                                                                          *}
{****************************************************************************}

constructor TDZVersionInfo.Create( AOwner: TComponent );
begin
  inherited Create(AOwner);
  FPInfoBuffer := nil;
  FileName := '';
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          Destroy()                                                *}
{*                                                                          *}
{*  Beschreibung:  Destruktor der TDZVersionInfo-Klasse                     *}
{*                                                                          *}
{****************************************************************************}

destructor TDZVersionInfo.Destroy;
begin
  if (FPInfoBuffer <> nil) then
    StrDispose(FPInfoBuffer);

  inherited Destroy;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          ReadVersionInfo()                                        *}
{*                                                                          *}
{*  Beschreibung:  Liest die Versionsinformationen der Datei <FileName>.    *}
{*                                                                          *}
{****************************************************************************}

procedure TDZVersionInfo.ReadVersionInfo;
var
  PFileName: PChar;
  InfoSize:  DWORD;
  Handle:    DWORD;
  Len:       UINT;
  P:         pointer;
begin
  FInfoAvailable := false;

  PFileName := StrAlloc(length(FFileName)+1);
  StrPCopy(PFileName,FFileName);

  InfoSize := GetFileVersionInfoSize(PFileName,Handle);

  if (InfoSize > 0) then begin
    if (FPInfoBuffer <> nil) then
      StrDispose(FPInfoBuffer);

    FPInfoBuffer := StrAlloc(InfoSize);

    if (GetFileVersionInfo(PFileName,Handle,InfoSize,FPInfoBuffer)) then begin
      FInfoAvailable := true;

      VerQueryValue(FPInfoBuffer,'\',P,Len);
      FFixedInfo := PVSFixedFileInfo(P)^;

      VerQueryValue(FPInfoBuffer,'\VarFileInfo\Translation',P,Len);
      FLanguageCode := PTranslation(P)^.Lang;
      FCharsetCode := PTranslation(P)^.Charset;
    end;
  end;

  StrDispose(PFileName);
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetStringFileInfo()                                      *}
{*                                                                          *}
{*  Beschreibung:  Ermittelt eine bestimmte String-Versionsinfo anhand      *}
{*                 des bergebenen Index.                                   *}
{*                                                                          *}
{*  Input:         Index - Index der String-Versionsinfo                    *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetStringFileInfo( Index: integer ): string;
var
  LangCharset: string;
  CommandBuf:  array [0..255] of char;
  P:           pointer;
  Len:         UINT;
begin
  Result := '';

  if (FInfoAvailable) then begin
    LangCharset := Format('%4.4x%4.4x',[FLanguageCode,FCharsetCode]);
    StrPCopy(CommandBuf,'\StringFileInfo\'+LangCharset+'\'+SFINames[Index]);

    if (VerQueryValue(FPInfoBuffer,CommandBuf,P,Len)) then
      Result := StrPas(PChar(P));
  end;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          SetFileName()                                            *}
{*                                                                          *}
{*  Beschreibung:  Setzt die Eigenschaft <FileName>.                        *}
{*                                                                          *}
{*  Input:         Value - Neuer Wert fr <FileName>                        *}
{*                                                                          *}
{****************************************************************************}

procedure TDZVersionInfo.SetFileName( Value: string );
begin
  if (csDesigning in ComponentState) then
    FFileName := Value
  else begin
    if (Value = '') then
      FFileName := Application.ExeName
    else
      FFileName := Value;

    ReadVersionInfo;
  end;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetFileVersionNumber()                                   *}
{*                                                                          *}
{*  Beschreibung:  Liefert die Dateiversion von <FileName> als Record.      *}
{*                                                                          *}
{*  Return:        Dateiversion                                             *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetFileVersionNumber: TVersionNumber;
begin
  if (FInfoAvailable) then begin
    Result.V1 := HiWord(FFixedInfo.dwFileVersionMS);
    Result.V2 := LoWord(FFixedInfo.dwFileVersionMS);
    Result.V3 := HiWord(FFixedInfo.dwFileVersionLS);
    Result.V4 := LoWord(FFixedInfo.dwFileVersionLS);
  end
  else begin
    Result.V1 := 0;
    Result.V2 := 0;
    Result.V3 := 0;
    Result.V4 := 0;
  end;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetProductVersionNumber()                                *}
{*                                                                          *}
{*  Beschreibung:  Liefert die Produktversion von <FileName> als Record.    *}
{*                                                                          *}
{*  Return:        Produktversion                                           *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetProductVersionNumber: TVersionNumber;
begin
  if (FInfoAvailable) then begin
    Result.V1 := HiWord(FFixedInfo.dwProductVersionMS);
    Result.V2 := LoWord(FFixedInfo.dwProductVersionMS);
    Result.V3 := HiWord(FFixedInfo.dwProductVersionLS);
    Result.V4 := LoWord(FFixedInfo.dwProductVersionLS);
  end
  else begin
    Result.V1 := 0;
    Result.V2 := 0;
    Result.V3 := 0;
    Result.V4 := 0;
  end;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetLanguageCode()                                        *}
{*                                                                          *}
{*  Beschreibung:  Liefert den Sprachen-Code von <FileName>.                *}
{*                                                                          *}
{*  Return:        Sprachen-Code                                            *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetLanguageCode: word;
begin
  if (FInfoAvailable) then
    Result := FLanguageCode
  else
    Result := 0;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetCharsetCode()                                         *}
{*                                                                          *}
{*  Beschreibung:  Liefert den Zeichensatz-Code von <FileName>.             *}
{*                                                                          *}
{*  Return:        Zeichensatz-Code                                         *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetCharsetCode: word;
begin
  if (FInfoAvailable) then
    Result := FCharsetCode
  else
    Result := 0;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetLanguage()                                            *}
{*                                                                          *}
{*  Beschreibung:  Liefert die Sprache von <FileName> als String.           *}
{*                                                                          *}
{*  Return:        Sprache                                                  *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetLanguage: string;
var
  Buf: array [0..255] of char;
begin
  if (FInfoAvailable) then begin
    VerLanguageName(FLanguageCode,Buf,sizeof(Buf));
    Result := StrPas(Buf);
  end
  else
    Result := '';
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetIsDebug()                                             *}
{*                                                                          *}
{*  Beschreibung:  Liefert den Zustand des Flags Debug von <FileName>.      *}
{*                                                                          *}
{*  Return:        Flag Debug                                               *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetIsDebug: boolean;
begin
  if (FInfoAvailable) then
    if (FFixedInfo.dwFileFlagsMask and VS_FF_DEBUG <> 0) then
      Result := (FFixedInfo.dwFileFlags and VS_FF_DEBUG <> 0)
    else
      Result := false
  else
    Result := false;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetIsPreRelease()                                        *}
{*                                                                          *}
{*  Beschreibung:  Liefert den Zustand des Flags PreRelease von             *}
{*                 <FileName>.                                              *}
{*                                                                          *}
{*  Return:        Flag PreRelease                                          *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetIsPreRelease: boolean;
begin
  if (FInfoAvailable) then
    if (FFixedInfo.dwFileFlagsMask and VS_FF_PRERELEASE <> 0) then
      Result := (FFixedInfo.dwFileFlags and VS_FF_PRERELEASE <> 0)
    else
      Result := false
  else
    Result := false;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetIsPrivateBuild()                                      *}
{*                                                                          *}
{*  Beschreibung:  Liefert den Zustand des Flags PrivateBuild von           *}
{*                 <FileName>.                                              *}
{*                                                                          *}
{*  Return:        Flag PrivateBuild                                        *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetIsPrivateBuild: boolean;
begin
  if (FInfoAvailable) then
    if (FFixedInfo.dwFileFlagsMask and VS_FF_PRIVATEBUILD <> 0) then
      Result := (FFixedInfo.dwFileFlags and VS_FF_PRIVATEBUILD <> 0)
    else
      Result := false
  else
    Result := false;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          GetIsSpecialBuild()                                      *}
{*                                                                          *}
{*  Beschreibung:  Liefert den Zustand des Flags SpecialBuild von           *}
{*                 <FileName>.                                              *}
{*                                                                          *}
{*  Return:        Flag SpecialBuild                                        *}
{*                                                                          *}
{****************************************************************************}

function TDZVersionInfo.GetIsSpecialBuild: boolean;
begin
  if (FInfoAvailable) then
    if (FFixedInfo.dwFileFlagsMask and VS_FF_SPECIALBUILD <> 0) then
      Result := (FFixedInfo.dwFileFlags and VS_FF_SPECIALBUILD <> 0)
    else
      Result := false
  else
    Result := false;
end;

{****************************************************************************}
{*                                                                          *}
{*  Name:          Register()                                               *}
{*                                                                          *}
{*  Beschreibung:  Registrierung der TDZVersionInfo-Klasse.                 *}
{*                                                                          *}
{****************************************************************************}

procedure Register;
begin
  RegisterComponents('DZ',[TDZVersionInfo]);
end;

end.
