unit exit1;

interface

uses

  Buttons,
  Classes,
  Controls,
  ExtCtrls,
  Forms,
  Graphics,
  IniFiles,
  Messages,
  ShellAPI,
  StdCtrls,
  SysUtils,
  Windows,

  DZVInfo;

const

  WM_ICONACTION = WM_USER+1;

type

  TMainForm = class(TForm)
    btnCancel: TBitBtn;
    btnOK: TBitBtn;
    edDate: TEdit;
    edTime: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Panel: TPanel;
    Timer: TTimer;
    VersionInfo: TDZVersionInfo;
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
  private
    FCommand: PChar;
    FCountDown: Boolean;
    FIconData: TNOTIFYICONDATA;
    FKind: Byte;
    FSeconds: Integer;
    FText: ARRAY[3..23] OF STRING;
    FTime1,FTime2,FTime3: TDateTime;
    procedure WMIconAction(var Msg: TMessage); message WM_ICONACTION;
  public
    Procedure CreateIcon(Icon: TIcon; Tip: String);
    procedure HideIt;
    Procedure KillIcon;
    procedure Reboot;
    procedure SetCountDown;
    procedure ShowIt;
    procedure ShutDown;
  end;

var

  MainForm: TMainForm;

function GetShutDownPrivilege: Boolean;

implementation

{$R *.DFM}

function GetShutDownPrivilege: Boolean;
var
  hToken    : THandle;
  tkp       : TTokenPrivileges;
  retlength : DWORD;
  Newt      : TTokenPrivileges;
begin
  Result := False;
  if OpenProcessToken(GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, hToken) then
  begin
    tkp.PrivilegeCount := 1;  // One to set
    // Get the LUID for shutdown privilege
    if LookupPrivilegeValue( nil, 'SeShutdownPrivilege',tkp.Privileges[0].Luid) then
    begin
      tkp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
      retlength:=0;
      // Get shutdown privilege for this process
      Result := AdjustTokenPrivileges(hToken, False, tkp, SizeOf(TTokenPrivileges), Newt, retlength);
    end;
    CloseHandle(hToken);
  end;
end; {GetShutDownPrivilege}

procedure TMainForm.FormCreate(Sender: TObject);
var
  Ini: TIniFile;
  CommandStr: String;
  Loop: Integer;
  Lng: String;
begin
  Ini:=TIniFile.Create(ChangeFileExt(paramstr(0),'.ini'));
  FSeconds:=Ini.ReadInteger('Options','Time',5)+1;
  CommandStr:=Ini.ReadString('Options','Command','');
  FKind:=Ini.ReadInteger('Options','Kind',3);
  GetMem(FCommand,Length(CommandStr)+1);
  StrPCopy(FCommand,CommandStr);
  Caption:=VersionInfo.ProductName+' v'+VersionInfo.ProductVersion+'  '+VersionInfo.LegalCopyright;
  Lng:=Ini.ReadString('Options','Language','');
  for Loop:=Low(FText) to High(FText) do
  begin
    FText[Loop]:=Ini.ReadString(Lng,IntToStr(Loop),'');
  end;
  label2.caption:=FText[16];
  btnCancel.caption:=FText[17];
  Ini.Free;
end;

procedure TMainForm.ShutDown; {Windows beenden}
begin
  panel.visible:=false;
  btnOK.visible:=false;
  btnCancel.top:=82;
  MainForm.tag:=2;
  MainForm.Height:=146;
  MainForm.top:=ROUND((GetDeviceCaps(GetDC(GetActiveWindow),VERTRES)-MainForm.height)/2);
  FCountDown:=false;
  TimerTimer(MainForm);
  showit;
  Timer.Enabled:=TRUE;
end;

procedure TMainForm.SetCountDown; {Countdown-Timer}
begin
  panel.visible:=true;
  btnOK.visible:=true;
  btnCancel.top:=112;
  btnCancel.left:=120;
  MainForm.tag:=2;
  MainForm.Height:=176;
  MainForm.width:=281;
  label1.caption:='';
  MainForm.top:=ROUND((GetDeviceCaps(GetDC(GetActiveWindow),VERTRES)-MainForm.height)/2);
  MainForm.left:=ROUND((GetDeviceCaps(GetDC(GetActiveWindow),HORZRES)-MainForm.width)/2);
  IF edDate.text='' THEN edDate.text:=DateToStr(Date);
  showit;
end;

procedure TMainForm.Reboot; {System neustarten}
begin
  panel.visible:=false;
  btnOK.visible:=false;
  btnCancel.top:=82;
  MainForm.tag:=3;
  MainForm.Height:=146;
  MainForm.top:=ROUND((GetDeviceCaps(GetDC(GetActiveWindow),VERTRES)-MainForm.height)/2);
  FCountDown:=false;
  TimerTimer(MainForm);
  showit;
  Timer.Enabled:=true;
end;

procedure TMainForm.WMIconAction(var Msg: TMessage);
begin
   with msg do
   begin
     if (msg=WM_ICONACTION) then
     begin
       case lParam of
         WM_LBUTTONUp: ShutDown;
         WM_MBUTTONUp: if FKind=3 then SetCountDown;
         WM_RBUTTONUp: if FKind=1 then SetCountDown else Reboot;
       end;
     end else
     begin
       if Msg in [WM_QUERYENDSESSION, WM_CLOSE, WM_QUIT, WM_DESTROY, WM_NCDESTROY, WM_USERCHANGED]
         then Result:=1
         else Result:=DefWindowProc(FIconData.Wnd, Msg, wParam, lParam);
     end;
  end;
end;

procedure TMainForm.CreateIcon(Icon: TIcon; Tip: String);
begin
  with FIconData do
  begin
    cbSize:=SizeOf(TNOTIFYICONDATA);
    wnd:=AllocateHWnd(WMIconAction);
    uID:=0;
    uFlags:=NIF_MESSAGE OR NIF_ICON OR NIF_TIP;
    hIcon:=Icon.Handle;
    StrPCopy(szTip,Tip);
    uCallbackMessage:=WM_ICONACTION;
  end;
  Shell_NotifyIcon(NIM_ADD,@FIconData);
end;

procedure TMainForm.KillIcon;
begin
  DeAllocateHWnd(FIconData.wnd);
  Shell_NotifyIcon(NIM_DELETE,@FIconData);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  KillIcon;
end;

procedure TMainForm.ShowIt;
begin
  ShowWindow(Self.Handle, SW_Show);
  ShowWindow(Application.Handle, SW_Show);
end;

procedure TMainForm.HideIt;
begin
  ShowWindow(Self.Handle, SW_Hide);
  ShowWindow(Application.Handle, SW_Hide);
end;

procedure TMainForm.FormPaint(Sender: TObject);
begin
  if tag=0 then
  begin
    timer.tag:=FSeconds;
    case FKind of
      1:   CreateIcon(Application.Icon,FText[18]+': '+FText[21]+' | '+FText[20]+': '+FText[22]);
      2:   CreateIcon(Application.Icon,FText[18]+': '+FText[21]+' | '+FText[20]+': '+FText[23]);
      else CreateIcon(Application.Icon,FText[18]+': '+FText[21]+' | '+FText[19]+': '+FText[22]+' | '+FText[20]+': '+FText[23]);
    end;
    hideit;
  end;
end;

procedure TMainForm.TimerTimer(Sender: TObject);
var
  HelpStr: String;
begin
  if FCountDown then
  begin
    if Date=FTime2
      then HelpStr:=FText[3]+edTime.Text+FText[4]
      else HelpStr:=FText[5]+edDate.Text+FText[6]+edTime.Text+FText[7];
    FIconData.hIcon:=MainForm.Icon.Handle;
    StrPCopy(FIconData.szTip,HelpStr);
    Shell_NotifyIcon(NIM_MODIFY,@FIconData);
    if now>FTime3 then
    begin
      panel.visible:=false;
      btnOK.visible:=false;
      btnCancel.top:=82;
      MainForm.Height:=146;
      MainForm.top:=ROUND((GetDeviceCaps(GetDC(GetActiveWindow),VERTRES)-MainForm.height)/2);
      FCountDown:=false;
      TimerTimer(MainForm);
      showit;
    end;
  end else
  begin
    timer.tag:=timer.tag-1;
    if MainForm.tag=2 then
    begin
      if timer.tag<1 then
      begin
        MainForm.bordericons:=[];
        label1.caption:=FText[8];
        MainForm.height:=72;
        label1.width:=400;
        label1.height:=32;
        btnCancel.top:=8;
        btnCancel.Hide;
      end else
      begin
        if timer.tag=1
          then label1.caption:=FText[9]
          else label1.caption:=FText[10]+IntToStr(timer.Tag)+FText[11];
        label1.width:=260;
      end;
      {$IFNDEF DEBUG}
      if timer.tag=0 then winEXEC(FCommand,0);
      if timer.tag<0 then
      begin
        Timer.Enabled:=false;
        ExitWindowsEx(EWX_Force or EWX_PowerOff,0);
      end;
      {$ENDIF}
    end;
    if MainForm.tag=3 then
    begin
      if timer.tag<1 then
      begin
        MainForm.bordericons:=[];
        label1.caption:=FText[12];
        MainForm.height:=72;
        label1.height:=32;
        label1.width:=444;
        btnCancel.top:=8;
        btnCancel.Hide;
      end else
      begin
        if timer.tag=1
          then label1.caption:=FText[13]
          else label1.caption:=FText[14]+IntToStr(timer.tag)+FText[15];
        label1.width:=307;
      end;
      {$IFNDEF DEBUG}
      if timer.tag=0 then winEXEC(FCommand,0);
      if timer.tag<0 then
      begin
        Timer.Enabled:=false;
        ExitWindowsEx(EWX_Force or EWX_Reboot,0);
      end;
      {$ENDIF}
    end;
    MainForm.width:=label1.width+20;
    MainForm.btnCancel.left:=ROUND((MainForm.width-label1.width)/2);
    MainForm.left:=ROUND((GetDeviceCaps(GetDC(GetActiveWindow),HORZRES)-MainForm.width)/2);
    btnCancel.left:=ROUND((MainForm.width-btnCancel.width)/2);
  end;
end;

procedure TMainForm.btnCancelClick(Sender: TObject);
begin
  FCountDown:=false;
  FIconData.hIcon:=Application.Icon.Handle;
    case FKind of
      1:   StrPCopy(FIconData.szTip,FText[18]+': '+FText[21]+' | '+FText[20]+': '+FText[22]);
      2:   StrPCopy(FIconData.szTip,FText[18]+': '+FText[21]+' | '+FText[20]+': '+FText[23]);
      else StrPCopy(FIconData.szTip,FText[18]+': '+FText[21]+' | '+FText[19]+': '+FText[22]+' | '+FText[20]+': '+FText[23]);
    end;
  Shell_NotifyIcon(NIM_MODIFY,@FIconData);
  timer.enabled:=FALSE;
  timer.tag:=FSeconds;
  hideit;
end;

procedure TMainForm.btnOKClick(Sender: TObject);
begin
  FTime1:=StrToTime(edTime.Text);
  FTime2:=StrToDate(edDate.Text);
  FTime3:=FTime1+FTime2;
  FCountDown:=true;
  TimerTimer(MainForm);
  Timer.Enabled:=TRUE;
  hideit;
end;

end.
